"""Provides Flask Forms related functionality."""
from wtforms import TextAreaField, DecimalField, SelectField, BooleanField
from wtforms.validators import DataRequired, NumberRange
from flask_wtf import FlaskForm
from osm2pgsql_tuner import config


def _get_pbf_gb_choices():
    """Prepares list of PBF size choices for drop-down menu.

    Returns
    -------------------------
    pbf_gb_choices : list
        list of tuples
    """
    pbf_gb_choices = list()
    for key, value in config.PBF_GB_SIZES.items():
        choice = (key,
                  f"{key} ({value['size_gb']} GB)")
        pbf_gb_choices.append(choice)

    return pbf_gb_choices


class Osm2pgsqlTunerInput(FlaskForm):
    system_ram_gb = SelectField('System RAM (GB)',
                                default=64,
                                choices=[4, 8, 16, 32, 64, 128, 256])
    pbf_gb_choices = _get_pbf_gb_choices()
    osm_pbf = SelectField('OSM PBF size (GB)',
                          default='North America',
                          choices=pbf_gb_choices)
    append = BooleanField('Use Append?', default=False)
    ssd = SelectField('Using SSD?', default='Yes, SSD',
                       choices=['Yes, SSD'])



"""osm2pgsq-tuner webapp routes.
"""
import logging
from flask import render_template, abort, request, redirect, jsonify
from osm2pgsql_tuner import app, forms, osm2pgsql, config

api_uri = '/api/v1'

@app.route('/', methods=['GET', 'POST'])
def view_root_path():
    app.logger.debug('Test messages from root path')
    form = forms.Osm2pgsqlTunerInput()

    if form.validate_on_submit():
        system_ram_gb = form.system_ram_gb.data
        osm_pbf = form.osm_pbf.data
        osm_pbf_details = config.PBF_GB_SIZES[osm_pbf]
        osm_pbf_gb = osm_pbf_details['size_gb']
        pbf_filename = osm_pbf_details['filename']
        append = form.append.data
        url_params = f'system_ram_gb={system_ram_gb}'
        url_params += f'&osm_pbf_gb={osm_pbf_gb}'
        url_params += f'&append={append}'
        url_params += f'&pbf_filename={pbf_filename}'
        url_params += f'&pgosm_layer_set=run-all'
        return redirect(f'/recommendation?{url_params}')

    return render_template('index.html', form=form)

def _get_api_params():
    try:
        system_ram_gb = float(request.args.get('system_ram_gb'))
        osm_pbf_gb = float(request.args.get('osm_pbf_gb'))
    except KeyError:
        abort(400)

    pbf_filename = request.args.get('pbf_filename')

    if pbf_filename is None:
        pbf_filename = config.DEFAULT_PBF_FILENAME

    append_raw = request.args.get('append')

    if append_raw == 'True':
        append = True
    else:
        append = False

    if request.args.get('pgosm_layer_set'):
        pgosm_layer_set = request.args.get('pgosm_layer_set')
    else:
        pgosm_layer_set = 'run-all'

    api_params = {'system_ram_gb': system_ram_gb,
                  'osm_pbf_gb': osm_pbf_gb,
                  'append': append,
                  'pbf_filename': pbf_filename,
                  'pgosm_layer_set': pgosm_layer_set}

    return api_params

def _get_recommendation(out_format):
    api_params = _get_api_params()

    rec = osm2pgsql.recommendation(system_ram_gb=api_params['system_ram_gb'],
                                   osm_pbf_gb=api_params['osm_pbf_gb'],
                                   append=api_params['append'],
                                   pgosm_layer_set=api_params['pgosm_layer_set'])
    cmd = rec.get_osm2pgsql_command(out_format=out_format,
                                    pbf_filename=api_params['pbf_filename'])
    rec_data = {'cmd': cmd, 'decisions': rec.decisions,
                'osm2pgsql_run_in_ram': rec.osm2pgsql_run_in_ram,
                'osm2pgsql_noslim_cache': rec.osm2pgsql_noslim_cache,
                'osm2pgsql_cache_max': rec.osm2pgsql_cache_max,
                'osm2pgsql_slim_cache': rec.osm2pgsql_slim_cache,
                'osm2pgsql_drop': rec.osm2pgsql_drop,
                'osm2pgsql_flat_nodes': rec.osm2pgsql_flat_nodes,
                'osm2pgsql_limited_ram': rec.osm2pgsql_limited_ram,
                'system_ram_gb': rec.system_ram_gb,
                'osm2pgsql_append': rec.append}

    return rec_data

@app.route('/recommendation')
def view_recommendation():
    rec_data = _get_recommendation(out_format='html')
    params = request.args
    return render_template('recommendation.html', rec_data=rec_data)


@app.route(api_uri)
def view_recommendation_api():
    rec_data = _get_recommendation(out_format='api')
    return jsonify(osm2pgsql= rec_data)


@app.route('/about')
def view_about():
    return render_template('about.html')




import os


APP_NAME = 'osm2pgsql-tuner'

CURR_PATH = os.path.abspath(os.path.dirname(__file__))
PROJECT_BASE_PATH = os.path.abspath(os.path.join(CURR_PATH, os.pardir))

try:
    LOG_PATH = os.environ['LOG_PATH']
except KeyError:
    LOG_PATH = PROJECT_BASE_PATH + '/webapp.log'


# Required for CSRF protection in Flask, please change to something secret!
try:
    APP_SECRET_KEY = os.environ['APP_SECRET_KEY']
except KeyError:
    ERR_MSG = '\nSECURITY WARNING: To ensure security please set the APP_SECRET_KEY'
    ERR_MSG += ' environment variable.\n'
    #LOGGER.warning(ERR_MSG)
    print(ERR_MSG)
    APP_SECRET_KEY = 'S$332sgaasgklMGSU89y2gslkmlLLd vlmssajfsdgGADAAJjd77j@neHMl'



try:
    APP_DEBUG_RAW = os.environ['APP_DEBUG']
    if APP_DEBUG_RAW == 'False':
        APP_DEBUG = False
    else:
        APP_DEBUG = True
except KeyError:
    APP_DEBUG = True

DEFAULT_PBF_FILENAME = 'colorado-latest'

PBF_GB_SIZES = {'Colorado': {'size_gb': 0.203, 'filename': 'colorado-latest'},
                'California': {'size_gb': 0.893, 'filename': 'california-latest'},
                'Norway': {'size_gb': 1, 'filename': 'norway-latest'},
                'Germany': {'size_gb': 3.4, 'filename': 'germany-latest'},
                'Africa': {'size_gb': 4.5, 'filename': 'africa-latest'},
                'North America': {'size_gb': 10.4, 'filename': 'north-america-latest'},
                'Europe': {'size_gb': 23.4, 'filename': 'europe-latest'},
                'Planet': {'size_gb': 59.0, 'filename': 'planet-latest'}
                }
"""Regional PBF sizes for reference based on details from
Geofabrik's download server on 5/15/2021.

Planet PBF size from https://planet.openstreetmap.org/pbf/ on 7/21/2021.
"""

FLAT_NODES_THRESHOLD_GB = 8.0



import logging
from flask import Flask
from osm2pgsql_tuner import config

# App settings
app = Flask(__name__)
app.config['DEBUG'] = config.APP_DEBUG
app.config['SECRET_KEY'] = config.APP_SECRET_KEY
app.config['WTF_CSRF_ENABLED'] = True

if config.APP_DEBUG:
    log_level = logging.DEBUG
else:
    log_level = logging.INFO

logging.basicConfig(filename=config.LOG_PATH,
                    level=log_level,
                    format=f'%(asctime)s %(levelname)s %(name)s %(threadName)s : %(message)s')


from osm2pgsql_tuner import routes
